@@ -15,7 +15,7 @@ import pylab
 import numpy as np
 import scipy.optimize
 import types
-from c_utils2d import polartransform, radintC,imageintC,azimintpixC, azimintqC, bin2D, calculateDmatrix
+from c_utils2d import polartransform, radintC,imageintC,azimintpixC, azimintqC, bin2D, calculateDmatrix, twodimfromonedim
 HC=12398.419 #Planck's constant times speed of light, in eV*Angstrom units
 
 def findbeam_gravity(data,mask):
@@ -56,7 +56,7 @@ def findbeam_gravity(data,mask):
     # indices where both nix and spamx is nonzero.
     goodx=((nix!=0) & (spamx!=0))
     # trim y, nix and spamx by goodx, eliminate invalid points.
-    y1=y[goodx]
+    #y1=y[goodx]
     nix=nix[goodx]
     spamx=spamx[goodx]
 
@@ -64,7 +64,7 @@ def findbeam_gravity(data,mask):
     niy=np.dot(data1.T,y).flatten()
     spamy=np.dot(data1.T,oney).flatten()
     goody=((niy!=0) & (spamy!=0))
-    x1=x[goody]
+#    x1=x[goody]
     niy=niy[goody]
     spamy=spamy[goody]
     # column coordinate of the center in each row will be contained in
@@ -119,6 +119,7 @@ def findbeam_slices(data,orig_initial,mask=None,maxiter=0):
         return ret
     orig=scipy.optimize.leastsq(targetfunc,np.array(orig_initial),args=(data,1-mask),maxfev=maxiter,epsfcn=0.0001)
     return orig[0]
+
 def findbeam_azimuthal(data,orig_initial,mask=None,maxiter=100,Ntheta=50,dmin=0,dmax=np.inf):
     """Find beam center using azimuthal integration
     
@@ -139,7 +140,6 @@ def findbeam_azimuthal(data,orig_initial,mask=None,maxiter=100,Ntheta=50,dmin=0,
             starting from 1
     """
     print "Finding beam (azimuthal), please be patient..."
-    orig=np.array(orig_initial)
     if mask is None:
         mask=np.ones(data.shape)
     def targetfunc(orig,data,mask):
@@ -154,6 +154,38 @@ def findbeam_azimuthal(data,orig_initial,mask=None,maxiter=100,Ntheta=50,dmin=0,
         return abs(p[0])
     orig1=scipy.optimize.fmin(targetfunc,np.array(orig_initial),args=(data,1-mask),maxiter=maxiter)
     return orig1
+
+def findbeam_azimuthal_fold(data,orig_initial,mask=None,maxiter=100,Ntheta=50,dmin=0,dmax=np.inf):
+    """Find beam center using azimuthal integration and folding
+    
+    Inputs:
+        data: scattering matrix
+        orig_initial: estimated value for x (row) and y (column)
+            coordinates of the beam center, starting from 1.
+        mask: mask matrix. If None, nothing will be masked. Otherwise it
+            should be of the same size as data. Nonzero means non-masked.
+        maxiter: maximum number of iterations for scipy.optimize.fmin
+        Ntheta: the number of theta points for the azimuthal integration.
+            Should be even!
+        dmin: pixels nearer to the origin than this will be excluded from
+            the azimuthal integration
+        dmax: pixels farther from the origin than this will be excluded from
+            the azimuthal integration
+    Output:
+        a vector of length 2 with the x and y coordinates of the origin,
+            starting from 1
+    """
+    print "Finding beam (azimuthal_fold), please be patient..."
+    if Ntheta%2:
+        raise ValueError('Ntheta should be even in function findbeam_azimuthal_fold()!')
+    if mask is None:
+        mask=np.ones(data.shape)
+    def targetfunc(orig,data,mask):
+        t,I,a=azimintpixC(data,None,orig,mask.astype('uint8'),Ntheta,dmin,dmax)
+        return np.sum((I[:Ntheta/2]-I[Ntheta/2:])**2)/Ntheta
+    orig1=scipy.optimize.fmin(targetfunc,np.array(orig_initial),args=(data,1-mask),maxiter=maxiter)
+    return orig1
+
 def findbeam_semitransparent(data,pri):
     """Find beam with 2D weighting of semitransparent beamstop area
 
@@ -167,20 +199,34 @@ def findbeam_semitransparent(data,pri):
     Outputs: bcx,bcy
         the x and y coordinates of the primary beam
     """
+    threshold=0.05
     print "Finding beam (semitransparent), please be patient..."
-    xmin=min([pri[0],pri[1]])
-    ymin=min([pri[2],pri[3]])
-    xmax=max([pri[0],pri[1]])
-    ymax=max([pri[2],pri[3]])
+#    xmin=min([pri[0],pri[1]])
+#    ymin=min([pri[2],pri[3]])
+#    xmax=max([pri[0],pri[1]])
+#    ymax=max([pri[2],pri[3]])
     C,R=np.meshgrid(np.arange(data.shape[1]),
                        np.arange(data.shape[0]))
-    indices=((C<=xmax) & (C>=xmin) & (R<=ymax) & (R>=ymin))
-    d=data[indices]
-    x=R[indices]
-    y=C[indices]
-    bcx=np.sum(d*x)/np.sum(d)
-    bcy=np.sum(d*y)/np.sum(d)
+    B=data[pri[2]:pri[3],pri[0]:pri[1]];
+    Ri=range(pri[2],pri[3])
+    Ci=range(pri[0],pri[1])
+    Ravg=np.sum(B,1)/len(Ri)
+    Cavg=np.sum(B,0)/len(Ci)
+    maxR=max(Ravg)
+    maxRpos=[i for i,avg in zip(Ri,Ravg) if avg==maxR][0]
+    Rmin=[i for i,avg in zip(Ri,Ravg) if (avg<Ravg[0]+(maxR-Ravg[0])*threshold) and (i<maxRpos)][-1]    
+    Rmax=[i for i,avg in zip(Ri,Ravg) if (avg<Ravg[-1]+(maxR-Ravg[-1])*threshold) and (i>maxRpos)][0]
+    maxC=max(Cavg)
+    maxCpos=[i for i,avg in zip(Ci,Cavg) if avg==maxC][0]
+    Cmin=[i for i,avg in zip(Ci,Cavg) if (avg<Cavg[0]+(maxC-Cavg[0])*threshold) and (i<maxCpos)][-1]    
+    Cmax=[i for i,avg in zip(Ci,Cavg) if (avg<Cavg[-1]+(maxC-Cavg[-1])*threshold) and (i>maxCpos)][0]
+    d=data[Rmin:Rmax,Cmin:Cmax]
+    x=np.arange(Rmin,Rmax)
+    y=np.arange(Cmin,Cmax)
+    bcx=np.sum(np.sum(d,1)*x)/np.sum(d)+1
+    bcy=np.sum(np.sum(d,0)*y)/np.sum(d)+1
     return bcx,bcy
+
 def azimintpix(data,error,orig,mask,Ntheta=100,dmin=0,dmax=np.inf):
     """Perform azimuthal integration of image.
 
@@ -207,7 +253,7 @@ def azimintpix(data,error,orig,mask,Ntheta=100,dmin=0,dmax=np.inf):
 
     # remove invalid pixels (masked or falling outside [dmin,dmax])
     valid=((mask==0)&(D<=dmax)&(D>=dmin))
-    d=D[valid]
+ #   d=D[valid]
     dat=data[valid]
     if error is not None:
         err=error[valid]
@@ -400,28 +446,6 @@ def radint(data,dataerr,energy,distance,res,bcx,bcy,mask,q=None,a=None,shutup=Tr
     
     return q,Intensity,Error,Area # return
     
-#def calculateDmatrix(mask,res,bcx,bcy):
-#    """Calculate distances of pixels from the origin
-#    
-#    Inputs:
-#        mask: mask matrix (only its shape is used)
-#        res: pixel size in mm-s. Can be a vector of length 2 or a scalar
-#        bcx: Beam center in pixels, in the row direction, starting from 1
-#        bcy: Beam center in pixels, in the column direction, starting from 1
-#        
-#    Output:
-#        A matrix of the shape of <mask>. Each element contains the distance
-#        of the centers of the pixels from the origin (bcx,bcy), expressed in
-#        mm-s.
-#    """
-#    if type(res)!=type([]) and type(res)!=type(()):
-#        res=[res]
-#    if len(res)<2:
-#        res=res*2
-#    Y,X=np.meshgrid(np.arange(mask.shape[1]),np.arange(mask.shape[0]));
-#    D=np.sqrt((res[0]*(X-bcx-1))**2+
-#                 (res[1]*(Y-bcy-1))**2)
-#    return D
 def qrangefrommask(mask,energy,distance,res,bcx,bcy,fullyunmasked=False):
     """Calculate q-range from mask matrix
     
@@ -447,30 +471,18 @@ def qrangefrommask(mask,energy,distance,res,bcx,bcy,fullyunmasked=False):
     D=calculateDmatrix(mask,res,bcx,bcy)
     dmin=np.nanmin(D[mask!=0])
     dmax=np.nanmax(D[mask!=0])
+    qmin=4*np.pi*np.sin(0.5*np.arctan(dmin/distance))*energy/HC
+    qmax=4*np.pi*np.sin(0.5*np.arctan(dmax/distance))*energy/HC
     Nq=np.ceil(dmax-dmin)
 
-    q0,I0,E0,A0=radintC(mask.astype(np.double),\
-                            np.ones(mask.shape,np.double),\
-                            energy,distance,\
-                            res,bcx,bcy,\
-                            np.zeros(mask.shape,np.uint8),q=None,\
-                            returnavgq=False)
     if fullyunmasked:
-        qmin=4*np.pi*np.sin(0.5*np.arctan(dmin/distance))*energy/HC
-        qmax=4*np.pi*np.sin(0.5*np.arctan(dmax/distance))*energy/HC
         # the smallest of the border elements in D
         Dbordermin=min(D[:,0].min(),D[0,:].min(),D[-1,:].min(),D[:,-1].min())
         qbordermin=4*np.pi*np.sin(0.5*np.arctan(Dbordermin/distance))*energy/HC
-        qmin1=q0[np.nonzero(I0==1)[0][0]] # find the first unmasked q-value
-        qmax1=q0[np.nonzero(I0==1)[0][-1]] # find the last unmasked q-value
-        qmax1=min(qmax1,qbordermin)
-    else:
-        qmin1=q0[np.nonzero(I0>0)[0][0]] # find the first unmasked q-value
-        qmax1=q0[np.nonzero(I0>0)[0][-1]] # find the last unmasked q-value
-#        print "qrangefrommask: old method - new method: qmin: ",qmin-qmin1,"; qmax:",qmax-qmax1
+        qmax=min(qmax,qbordermin)
 
     
-    return qmin1,qmax1,Nq
+    return qmin,qmax,Nq
 def calculate2dfrom1d(q,I,Nx,Ny,bcx,bcy,dist,energy,pixelsize,noise=False):
     """Create a 2D scattering image from a scattering curve.
     
@@ -515,7 +527,7 @@ def calculatetwothetamatrix(mat,dist,res,bcx,bcy):
         a matrix of the same shape as mat, containing twotheta values for
             the pixels.
     """
-    return np.arctan(calculateDmatrix(mat,res,bcx,bcy)/dist)
+    return np.arctan(calculateDmatrix(mat.astype(np.uint8),res,bcx,bcy)/dist)
 def calculateqmatrix(mat,dist,energy,res,bcx,bcy):
     """Calculate q values for detector pixels
     
@@ -532,4 +544,4 @@ def calculateqmatrix(mat,dist,energy,res,bcx,bcy):
             individual pixels.
     """
     return 4*np.pi*np.sin(0.5*np.arctan(calculateDmatrix(mat,res,bcx,bcy)/dist))*energy/HC
-    
\ No newline at end of file
+    
